<?php
class Vendorfuel_Templates {

	private static $instance = null;
	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new Vendorfuel_Templates();
		}
		return self::$instance;
	}
	protected $utilities;
	public function __construct() {
		$this->utilities = Vendorfuel_Utilities::get_instance();

        if ( ! function_exists( 'WP_Filesystem' ) ) {
            require_once ABSPATH . 'wp-admin/includes/file.php';
        }

        WP_Filesystem();

		//custom posts for SEO/Frontend deployment
		add_action( 'init', array( $this, 'create_post_types' ) );
		add_filter( 'comments_open', array( $this, 'my_prefix_comments_open' ), 10, 2 );
		add_action( 'the_post', array( $this, 'vf_custom_post_types_remove_rss_feed' ) );

		//templates
		add_shortcode( 'vf-template', array( $this, 'vendorfuel_template_shortcode' ) );
		add_action( 'permalink_structure_changed', array( $this, 'permalink_template_map_update' ) );

		//yoast custom variable
		add_action( 'wpseo_register_extra_replacements', array( $this, 'register_custom_yoast_variables' ), 10, 0 );

		//Handle legacy menu nav / footer implementation
		add_action( 'wp_body_open', array( $this, 'vendorfuel_menu_nav_init' ) );
		add_action( 'wp_footer', array( $this, 'vendorfuel_footer_init' ) );

		//template map permalink updates
		add_action( 'save_post',  array( $this,  'permalink_template_map_update') , 10, 1 );

		//insert required helper component
		add_action('wp_body_open', function(){ 
				echo '<vf-helper></vf-helper>';
			} 
		);
	}

	public function shortcode( $name ) {
		return do_shortcode( $this->utilities->get_template_content( $name ) );
	}

	/**
	 * Processes vf template shortcode params and replaces content accordingly.
	 *
	 * @param array $params The params passed in from the shortcode
	 * 
	 * @return string The html content of the template after params processed.
	 */
	public function vendorfuel_template_shortcode( $params ) {
		if ( isset( $params['name'] ) ) {
			$content = $this->shortcode( "{$params['name']}" );

			if ( isset( $params['product_id'] ) ) {
				return str_replace( 'ID_OF_PRODUCT', $params['product_id'], $content );
			}

			if ( isset( $params['cat_id'] ) ) {
				$content = str_replace( 'ID_OF_CATALOG', $params['cat_id'], $content );
				return str_replace( 'TYPE_OF_ID', 'CATEGORY', $content );
			}

			if ( isset( $params['collection_id'] ) ) {
				$content = str_replace( 'ID_OF_CATALOG', $params['collection_id'], $content );
				return str_replace( 'TYPE_OF_ID', 'COLLECTION', $content );
			}

			if ( isset( $params['mfg_id'] ) ) {
				$content = str_replace( 'ID_OF_CATALOG', $params['mfg_id'], $content );
				return str_replace( 'TYPE_OF_ID', 'MANUFACTURER', $content );
			}

			if ( isset( $params['customer_role'] ) ) {
				return str_replace( 'CUSTOMER_ROLE', $params['customer_role'], $content );
			}

			return $content;
		}
		return '';
	}

	public function vendorfuel_menu_nav_init() {
		//initialize the menu nav template into the WordPress header
		$store = get_option( 'vendorfuel-store-options' );
		if ( true === $store->options->{'Load Menu Nav Template'} ) {
			echo do_shortcode( '[vf-template name="menu-nav"]' );
		}
	}

	public function vendorfuel_footer_init() {
		$store = get_option( 'vendorfuel-store-options' );
		if ( true === $store->options->{'Load Footer Template'} ) {
			echo do_shortcode( '[vf-template name="footer"]' );
		}
	}

	public function vendorfuel_ccpa_init() {
		$store = get_option( 'vendorfuel-store-options' );
		if ( true === $store->options->{'Load CCPA Template'} ) {
			echo do_shortcode( '[vf-template name="ccpa"]' );
		}
	}

	public function my_prefix_comments_open( $open, $post_id ) {
		$post_type = get_post_type( $post_id );
		// allow comments for built-in "post" post type
		if ( 'post' === $post_type || 'vf-product' === $post_type || 'vf-category' === $post_type ) {
			return true;
		}
		// disable comments for any other post types
		return false;
	}

	// Our custom post type function
	public function create_post_types() {
		$general_settings = get_option( 'vendorfuel-general-settings' );
		register_post_type(
			'vf-product',
			// CPT Options
			array(
				'labels'          => array(
					'name'						=> __( 'VendorFuel Product Pages' ),
					'singular_name'				=> __( 'Product Page' ),
					'add_new_item'				=> __( 'Add New Product Page' ),
					'edit_item'					=> __( 'Edit Product Page' ),
					'new_item'					=> __( 'New Product Page' ),
					'view_item'					=> __( 'View Product Page' ),
					'view_items'				=> __( 'View Product Pages' ),
					'search_items'				=> __( 'Search Product Pages' ),
					'not_found'					=> __( 'No Product Pages found.' ),
					'not_found_in_trash'		=> __( 'No Product Pages found in Trash.' ),
					'all_items'					=> __( 'All Product Pages' ),
					'archives'					=> __( 'Product Page Archives' ),
					'attributes'				=> __( 'Product Page Attributes' ),
					'insert_into_item'			=> __( 'Insert into product page' ),
					'uploaded_to_this_item'		=> __( 'Uploaded to this product page' ),
					'menu_name'					=> __( 'VF Product Pages' ),
					'filter_items_list'			=> __( 'Filter product pages list' ),
					'items_list_navigation'		=> __( 'Product pages list navigation' ),
					'items_list'				=> __( 'Product pages list' ),
					'item_published'			=> __( 'Product Page published.' ),
					'item_published_privately'	=> __( 'Product Page published privately.' ),
					'item_reverted_to_draft'	=> __( 'Product Page reverted to draft.' ),
					'item_scheduled'			=> __( 'Product Page scheduled.' ),
					'item_updated'				=> __( 'Product Page updated.' ),
					'item_link'					=> __( 'Product Page Link.' ),
					'item_link_description'		=> __( 'A link to a product page.' ),
				),
				'public'          => true,
				'has_archive'     => false,
				'rewrite'         => array( 'slug' => ( isset( $general_settings->product_slug ) && '' !== $general_settings->product_slug ) ? $general_settings->product_slug : 'products' ),
				'supports'        => array( 'editor', 'title', 'custom-fields' ),
				'show_in_rest'    => true,
				'capability_type' => 'post',
			)
		);

		register_post_type(
			'vf-category',
			// CPT Options
			array(
				'labels'          => array(
					'name'						=> __( 'VendorFuel Category Pages' ),
					'singular_name'				=> __( 'Category Page' ),
					'add_new_item'				=> __( 'Add New Category Page' ),
					'edit_item'					=> __( 'Edit Category Page' ),
					'new_item'					=> __( 'New Category Page' ),
					'view_item'					=> __( 'View Category Page' ),
					'view_items'				=> __( 'View Category Pages' ),
					'search_items'				=> __( 'Search Category Pages' ),
					'not_found'					=> __( 'No Category Pages found.' ),
					'not_found_in_trash'		=> __( 'No Category Pages found in Trash.' ),
					'all_items'					=> __( 'All Category Pages' ),
					'archives'					=> __( 'Category Page Archives' ),
					'attributes'				=> __( 'Category Page Attributes' ),
					'insert_into_item'			=> __( 'Insert into category page' ),
					'uploaded_to_this_item'		=> __( 'Uploaded to this category page' ),
					'menu_name'					=> __( 'VF Category Pages' ),
					'filter_items_list'			=> __( 'Filter category pages list' ),
					'items_list_navigation'		=> __( 'Category pages list navigation' ),
					'items_list'				=> __( 'Category pages list' ),
					'item_published'			=> __( 'Category Page published.' ),
					'item_published_privately'	=> __( 'Category Page published privately.' ),
					'item_reverted_to_draft'	=> __( 'Category Page reverted to draft.' ),
					'item_scheduled'			=> __( 'Category Page scheduled.' ),
					'item_updated'				=> __( 'Category Page updated.' ),
					'item_link'					=> __( 'Category Page Link.' ),
					'item_link_description'		=> __( 'A link to a category page.' ),
				),
				'public'          => true,
				'has_archive'     => false,
				'rewrite'         => array( 'slug' => ( isset( $general_settings->cat_slug ) && '' !== $general_settings->cat_slug ) ? $general_settings->cat_slug : 'categories' ),
				'supports'        => array( 'editor', 'title', 'custom-fields' ),
				'show_in_rest'    => true,
				'capability_type' => 'post',
			)
		);
	}

	public function get_metadesc() {
		global $post;
		substr( get_post_meta( $post->ID, 'vfmetadesc', true ), 0, 155 );
	}

	// define the action for register yoast_variable replacments
	public function register_custom_yoast_variables() {
		if ( function_exists( 'wpseo_register_var_replacement' ) ) {
			wpseo_register_var_replacement( '%%vfmetadesc%%', array( $this, 'get_metadesc' ), 'advanced', 'VF Meta Description Field' );
		}
	}

	public function vf_custom_post_types_remove_rss_feed() {
		if ( 'vf-product' === get_post_type() || 'vf-category' === get_post_type() ) {
			remove_action( 'wp_head', 'feed_links_extra', 3 );
			remove_action( 'wp_head', 'feed_links', 2 );
		}
	}

	public function get_templates_override() {
		global $wp_filesystem;
		$templates = array();
		$dir       = glob( VENDORFUEL_CONTENT_DIR . '/frontend/html/*.{html}', GLOB_BRACE );

		foreach ( $dir as $file ) {
			$templates[] = (object) array(
				'name'    => basename( $file ),
				'content' => $wp_filesystem->get_contents( $file ),
			);
		}

		$response              = array();
		$response['templates'] = (object) $templates;
		$response['errors']    = array();
		return $response;
	}

	public function update_template_override( WP_REST_Request $request ) {
		global $wp_filesystem;

		$filename = VENDORFUEL_CONTENT_DIR . '/frontend/html/' . $request['name'];

		if ( $wp_filesystem->put_contents( $filename, $request['content'] ) ) {
			$response                  = array();
			$response['notifications'] = array( "Template override {$request['name']} updated." );
			$response['errors']        = array();
			return $response;
		}

		$response                  = array();
		$response['notifications'] = array();
		$response['errors']        = array( "Template override {$request['name']} failed." );
		return $response;

	}


    public function delete_template_override( WP_REST_Request $request ) {
        global $wp_filesystem;
        $request_data = json_decode($request->get_body(),true);
        $filename = VENDORFUEL_CONTENT_DIR . '/frontend/html/' . $request_data['name'];

        if ( $wp_filesystem->exists($filename) && $wp_filesystem->delete($filename) ) {
            $response                  = array();
            $response['notifications'] = array( "Template override {$request_data['name']} deleted." );
            $response['errors']        = array();
            return $response;
        }

        $response                  = array();
        $response['notifications'] = array();
        $response['errors']        = array( "Template override {$request['name']} delete failed." );
        return $response;

    }

	/**
	 * When permalinks setting is updated the template map urls will be updated.
	 *
	 */
	public function permalink_template_map_update() {
		//retrieve current anguledit settings where template_map is stored
		$anguledit_settings = get_option( 'vendorfuel-anguledit-settings' );

		if ( isset( $anguledit_settings->template_map ) ) {
			foreach ( $anguledit_settings->template_map as $slug => $template ) {
				$template->url                             = get_permalink( $template->id );
				$anguledit_settings->template_map->{$slug} = $template;
			}
			update_option( 'vendorfuel-anguledit-settings', $anguledit_settings );
		}
	}

	/**
	 * When a page/post is updated the template map will be searched for this post ID and url updated accordingly.
	 *
	 * @param integer $post_ID The ID of the post/page that was updated.
	 */
	public function update_template_map_links( $post_ID ) {
		//retrieve current anguledit settings where template_map is stored
		$anguledit_settings = get_option( 'vendorfuel-anguledit-settings' );

		if(isset($anguledit_settings->template_map)){
			//loop through template_map
			foreach($anguledit_settings->template_map as $index => $template){
				//match updated post ID with template_map post IDs
				if($template->id === $post_ID){
					//if matched
					//get new permalink of the updated post
					$template->url = get_permalink($post_ID);
					//store updated mapped template to template_map
					$anguledit_settings->template_map->{$index} = $template;
				}
			}
		}

		//persist updated template map
		update_option( 'vendorfuel-anguledit-settings', $anguledit_settings);
	}
}
